<?php
namespace Ak\Orm\Service;

use Doctrine\Common\Annotations\Reader;
use Doctrine\ORM\Mapping\Driver\AttributeDriver;
use Doctrine\Persistence\Mapping\Driver\MappingDriver;
use Doctrine\Persistence\Mapping\Driver\MappingDriverChain;
use Doctrine\ORM\Mapping\Driver\AnnotationDriver;
use Gedmo\DoctrineExtensions;
use Interop\Container\ContainerInterface;
use Laminas\ServiceManager\Factory\FactoryInterface;

/**
 * Class MappingDriverFactory
 *
 * @package Ak\Orm\Service
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/orm
 */
final class MappingDriverFactory implements FactoryInterface
{
    /**
     * Create an object
     *
     * @param  ContainerInterface $container
     * @param  string $requestedName
     * @param  null|array $options
     *
     * @return MappingDriver
     */
    public function __invoke(ContainerInterface $container, $requestedName, array $options = null)
    {
        $config = $container->get('config')['orm'] ?? [];
        $driver = new MappingDriverChain();
        $reader = $container->get('orm.annotation_reader');

        $this->injectGedmoExtensions($driver, $reader);

        $allPaths = $config['entity_paths'] ?? [];
        $attributeNamespaces = $config['attribute_namespaces'] ?? [];

        foreach ($allPaths as $namespace => $paths) {
            if (in_array($namespace, $attributeNamespaces)) {
                $driver->addDriver(
                    new AttributeDriver((array) $paths),
                    $namespace
                );
            } else {
                $driver->addDriver(
                    new AnnotationDriver($reader, (array) $paths),
                    $namespace
                );
            }
        }

        return $driver;
    }

    /**
     * @param MappingDriverChain $driver
     * @param Reader $reader
     */
    private function injectGedmoExtensions(MappingDriverChain $driver, Reader $reader): void
    {
        DoctrineExtensions::registerAbstractMappingIntoDriverChainORM($driver, $reader);
    }
}
